package com.winhc.bigdata.udf;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.odps.udf.UDF;
import com.winhc.bigdata.utils.TitleUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import static cn.hutool.crypto.SecureUtil.md5;
import static com.winhc.bigdata.utils.CompanyUtils.cleanup;

/**
 * @Author: π
 * @Description: 标题合并
 */
public class case_title extends UDF {


    public String evaluate(String ygNameList, String bgNameList, String labelList, String caseReason, String caseNoList) {


        List<String> yg1 = convertNameList(ygNameList);
        List<String> bg1 = convertNameList(bgNameList);
        List<String> label1 = toList(labelList);
        List<String> caseno1 = toList(caseNoList);

        String title = TitleUtil.convertTitle(yg1, bg1, label1, caseReason, caseno1, null);

        return md5(cleanup(title));

    }

    public String evaluate(String ygNameList, String bgNameList, String labelList, String caseReason, String caseNoList, String allNameList) {


        List<String> yg1 = convertNameList(ygNameList);
        List<String> bg1 = convertNameList(bgNameList);
        List<String> all1 = convertNameList(allNameList);
        List<String> label1 = toList(labelList);
        List<String> caseno1 = toList(caseNoList);

        String title = TitleUtil.convertTitle(yg1, bg1, label1, caseReason, caseno1, all1);

        return md5(cleanup(title));

    }

    public String evaluate(String ygNameList, String bgNameList, String labelList, String caseReason, String caseNoList, String allNameList, String id) {


        List<String> yg1 = convertNameList(ygNameList);
        List<String> bg1 = convertNameList(bgNameList);
        List<String> all1 = convertNameList(allNameList);
        List<String> label1 = toList(labelList);
        List<String> caseno1 = toList(caseNoList);

        //没有原被告，排除系列案件
        if (CollectionUtils.isEmpty(yg1) || CollectionUtils.isEmpty(bg1)) {
            return id;
        }
        String title = TitleUtil.convertTitle(yg1, bg1, label1, caseReason, caseno1, all1);

        return md5(cleanup(title));

    }

    public List<String> convertNameList(String nameList) {
        return Optional.ofNullable(JSON.parseArray(nameList))
                .orElse(new JSONArray())
                .toJavaList(JSONObject.class)
                .stream().map(x -> x.getString("name"))
                .filter(x -> !StringUtils.isBlank(x))
                .distinct()
                .sorted()
                .collect(Collectors.toList());
    }

    public List<String> toList(String names) {
        return Optional.ofNullable(JSON.parseArray(names))
                .orElse(new JSONArray())
                .toJavaList(String.class)
                .stream()
                .filter(x -> !StringUtils.isBlank(x))
                .distinct().collect(Collectors.toList());
    }


    public static void main(String[] args) {
        String ygNameList = null;
        //String ygNameList = "[{\"lawyer_info\":[{\"lawfirm_id\":\"442ae2388d82bc6c518c1d39edef9628\",\"lawfirm_name\":\"广东瀛真律师事务所\",\"lawyer_id\":\"dd727883841cece61e09fd12f5c9aaff\",\"lawyer_name\":\"李雨蒙\"},{\"lawfirm_id\":\"442ae2388d82bc6c518c1d39edef9628\",\"lawfirm_name\":\"广东瀛真律师事务所\",\"lawyer_id\":\"b313fb9b61ef7f0156362e0d7548d113\",\"lawyer_name\":\"翁艺珊\"}],\"litigant_id\":\"56824a4671c1d05567ca10516e0a033d\",\"name\":\"中国建设银行股份有限公司广州从化支行\",\"party_title\":\"原告\"}]";
        String bgNameList = "[{\"lawyer_info\":[],\"litigant_id\":\"\",\"name\":\"余国春\",\"party_title\":\"被告\"},{\"lawyer_info\":[],\"litigant_id\":\"\",\"name\":\"郑志华\",\"party_title\":\"被告\"}]";
        String labelList = "[\"开庭公告\",\"限制高消费\",\"被执行人\",\"裁判文书\"]";
        String caseReason = "金融借款合同纠纷";
        String caseNoList = "[\"(2021)粤0117民初8876号\",\"(2023)粤0117执4323号\"]";
        case_title n = new case_title();
        System.out.println(n.evaluate(ygNameList, bgNameList, labelList, caseReason, caseNoList, bgNameList));
        System.out.println();

    }

}


