package com.winhc.bigdata.udf;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.JSONPath;
import com.aliyun.odps.udf.UDF;
import com.aliyun.odps.utils.StringUtils;
import com.google.common.collect.ImmutableMap;
import com.winhc.bigdata.bean.CaseInfo;
import com.winhc.bigdata.bean.CaseInfoName;
import com.winhc.bigdata.bean.BaseCaseInfo;
import com.winhc.bigdata.udf.justice.CaseNoPartyTitleParseUtilCopy;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Author: π
 * @Description: 数据获取原被告
 */
public class get_names extends UDF {

    private static final String yg_key = "0";
    private static final Map<String, String> maping = ImmutableMap.of("0", "原告", "1", "被告");


    public List<String> evaluate(String info) {

        if (StringUtils.isBlank(info)) {
            return new ArrayList<>();
        }
        //合并title
        List<BaseCaseInfo> datas = JSON.parseArray(info).toJavaList(CaseInfo.class).stream().map(d -> {
            BaseCaseInfo c0 = new BaseCaseInfo();
            c0.setFlag(d.getFlag());
            c0.setDetail_id(d.getDetail_id());

            c0.setCase_no(d.getCase_no());
            c0.setCase_reason(d.getCase_reason());

            JSONArray party_title = JSON.parseArray(d.getParty_title());
            c0.setParty_title(party_title);

            //合并title
            JSONArray yg_name = mergeTitle(JSON.parseArray(d.getYg_name()), party_title, "0", d.getCase_no());
            JSONArray bg_name = mergeTitle(JSON.parseArray(d.getBg_name()), party_title, "1", d.getCase_no());

            c0.setYg_name(yg_name);
            c0.setBg_name(bg_name);
            return c0;
        }).collect(Collectors.toList());

        //获取原被告
        CaseInfoName names = getNames(datas);

        //增加律师信息
        Map<String, JSONArray> m = mergeLawyers(datas);
        List<JSONObject> yg_name = addLawyer(names.getYg_name(), m);
        List<JSONObject> bg_name = addLawyer(names.getBg_name(), m);

        return Arrays.asList(JSON.toJSONString(yg_name), JSON.toJSONString(bg_name));
    }

    public CaseInfoName getNames(List<BaseCaseInfo> datas) {
        CaseInfoName outName = new CaseInfoName();

        //一审优先
        BaseCaseInfo case1 = datas.stream()
                .filter(c -> c.getCase_no() != null && c.getCase_no().contains("初"))
                .max(Comparator.comparing(BaseCaseInfo::getWeightV2))
                .orElse(null);

        if (case1 != null) {
            outName.setYg_name(case1.getYg_name().toJavaList(JSONObject.class));
            outName.setBg_name(case1.getBg_name().toJavaList(JSONObject.class));
            return outName;
        }

        //二审阶段，有文书判断
        BaseCaseInfo case2 = datas.stream()
                .filter(c -> c.getCase_no() != null && (c.getCase_no().contains("终") || c.getCase_no().contains("再")) && c.getFlag().equals("0"))
                .max(Comparator.comparing(BaseCaseInfo::getWeightV2))
                .orElse(null);

        if (case2 != null) {
            List<JSONArray> collect = Stream.of(case2.getYg_name(),
                            case2.getBg_name())
                    .collect(Collectors.toList());
            JSONArray allNames = mergeAllName(collect);
            Map<String, String> collect1 = Optional.ofNullable(case2.getParty_title())
                    .orElse(new JSONArray())
                    .toJavaList(JSONObject.class)
                    .stream()
                    .peek(t -> {
                        String partyTitle = t.getString("party_title");
                        String type = "n";
                        if (partyTitle != null) {
                            if (partyTitle.contains("原审") || partyTitle.contains("一审")) {
                                if (partyTitle.contains("被告")) {
                                    type = "b";
                                } else if (partyTitle.contains("原告")) {
                                    type = "y";
                                }
                            }

                        }

                        t.put("type", type);
                    })
                    .collect(Collectors.toMap(t -> t.getString("name"), t -> t.getString("type"), (n, o) -> n));

            List<JSONObject> yg_name = new ArrayList<>();
            List<JSONObject> bg_name = new ArrayList<>();
            allNames.toJavaList(JSONObject.class).forEach(x -> {
                String name = x.getString("name");
                if (collect1.containsKey(name)) {
                    if ("y".equals(collect1.get(name))) {
                        yg_name.add(x);
                    } else if ("b".equals(collect1.get(name))) {
                        bg_name.add(x);
                    }
                }
            });

            if (!yg_name.isEmpty() || !bg_name.isEmpty()) {
                outName.setYg_name(yg_name);
                outName.setBg_name(bg_name);
                return outName;
            }

        }

        List<BaseCaseInfo> reasons = datas.stream()
                .filter(c -> c.getCase_reason() != null
                        && (c.getCase_reason().contains("劳动争议")
                        || c.getCase_reason().contains("人事争议")))
                .collect(Collectors.toList());

        //如果没有文书，排除劳动争议
        if (reasons.size() > 0) {
            return outName;
        }

        //有执行阶段-原被告取集合
        List<BaseCaseInfo> zxr_list = datas.stream()
                .filter(c -> {
                    //String flag = c.getFlag();
                    return c.getCase_no() != null && c.getCase_no().contains("执");
                    //return zxrList.contains(flag);
                }).collect(Collectors.toList());

        if (zxr_list.size() > 0) {
            //原告集合
            JSONArray allYgNames = mergeAllName(zxr_list.stream().map(BaseCaseInfo::getYg_name)
                    .filter(Objects::nonNull)
                    .collect(Collectors.toList()));

            //被告集合
            JSONArray allBgNames = mergeAllName(zxr_list.stream().map(BaseCaseInfo::getBg_name)
                    .filter(Objects::nonNull)
                    .collect(Collectors.toList()));

            if (!allYgNames.isEmpty() || !allBgNames.isEmpty()) {
                outName.setYg_name(allYgNames.toJavaList(JSONObject.class));
                outName.setBg_name(allBgNames.toJavaList(JSONObject.class));
                return outName;
            }
        }

        return outName;
    }

    public static JSONArray mergeAllName(Collection<JSONArray> names) {
        if (null == names) return new JSONArray();
        HashMap<String, Map<String, String>> m = new HashMap<>();
        names.stream()
                .filter(Objects::nonNull).forEach(x -> parse(x, m));
        return new JSONArray(new ArrayList<>(m.values()));
    }

    public static void parse(JSONArray name, HashMap<String, Map<String, String>> m) {
        if (null == name || null == m) return;
        name.forEach(x -> {
            JSONObject z = (JSONObject) x;
            String name1 = z.getString("name");
            String litigant_id1 = z.getString("litigant_id");
            if (org.apache.commons.lang3.StringUtils.isNotBlank(name1) && (org.apache.commons.lang3.StringUtils.isNotBlank(litigant_id1) || !m.containsKey(name1))) {
                m.put(name1, (Map) z);
            }
        });

    }

    public String parse(String json, String json_path) {
        Object re = JSONPath.eval(JSON.parse(json), json_path);
        if (re == null) {
            return null;
        }
        return re.toString();
    }

    public static JSONArray mergeTitle(JSONArray name, JSONArray party_title, String idType, String case_no) {
        JSONArray res = new JSONArray();
        if (name == null || name.isEmpty()) return res;
        Map<String, String> pmap = Optional.ofNullable(party_title)
                .orElse(new JSONArray())
                .stream().map(v -> {
                    JSONObject d = (JSONObject) v;
                    String name1 = getKey(d, "name");
                    String party_title1 = getKey(d, "party_title");
                    if (StringUtils.isNotBlank(name1) && StringUtils.isNotBlank(party_title1)) {
                        return Arrays.asList(name1, party_title1);
                    }
                    return null;
                }).filter(Objects::nonNull)
                .collect(Collectors.toMap(t -> t.get(0), t -> t.get(1), (k1, k2) -> k1));
        //String defult_title = getTitle(idType, case_no);
        String defult_title = getTitleV2(idType, case_no, name);
        List<JSONObject> collect = name.stream().map(v -> {
            JSONObject j = new JSONObject();
            JSONObject d = (JSONObject) v;
            String name1 = getKey(d, "name");
            String litigant_id1 = getKey(d, "litigant_id");
            if (null != d && d.containsKey("lawyer_info")) {
                JSONArray lawyer_info = d.getJSONArray("lawyer_info");
                j.put("lawyer_info", lawyer_info);
            }
            j.fluentPut("name", name1)
                    .fluentPut("litigant_id", litigant_id1)
                    .fluentPut("party_title", pmap.getOrDefault(name1, defult_title));
            return j;
        }).collect(Collectors.toList());
        res.addAll(collect);
        return res;
    }


    public List<String> evaluate(String case_no, String yg_name, String bg_name) {
        if (StringUtils.isBlank(case_no) || (StringUtils.isBlank(yg_name) && StringUtils.isBlank(bg_name))) {
            return new ArrayList<>();
        }
        String yg_title_v2 = getTitleV2("0", case_no, JSON.parseArray(yg_name));
        String bg_title_v2 = getTitleV2("1", case_no, JSON.parseArray(bg_name));
        String yg_title = getTitle("0", case_no);
        if (StringUtils.isBlank(yg_name) || JSON.parseArray(yg_name).isEmpty()) {
            yg_title = null;
            yg_title_v2 = null;
        }
        String bg_title = getTitle("1", case_no);
        if (StringUtils.isBlank(bg_name) || JSON.parseArray(bg_name).isEmpty()) {
            bg_title = null;
            bg_title_v2 = null;
        }

        JSONArray yg_name_re = mergeTitle(JSON.parseArray(yg_name), new JSONArray(), "0", case_no);
        JSONArray bg_name_re = mergeTitle(JSON.parseArray(bg_name), new JSONArray(), "1", case_no);

        return Arrays.asList(yg_title, bg_title, yg_title_v2, bg_title_v2,JSON.toJSONString(yg_name_re),JSON.toJSONString(bg_name_re));
    }

    public static String getTitle(String flag, String case_no) {
        if (com.aliyun.odps.utils.StringUtils.isBlank(case_no)) return maping.get(flag);
        if (case_no.contains("执")) {
            return yg_key.equals(flag) ? "申请执行人" : "被执行人";
        } else if (case_no.contains("终")) {
            return yg_key.equals(flag) ? "上诉人" : "被上诉人";
        } else if (case_no.contains("行终")) {
            return yg_key.equals(flag) ? "上诉人" : "被上诉人";
        } else if (case_no.contains("刑初")) {
            return yg_key.equals(flag) ? "原告" : "被告";
        } else if (case_no.contains("民申")) {
            return yg_key.equals(flag) ? "再审申请人" : "被申请人";
        } else if (case_no.contains("财保")) {
            return yg_key.equals(flag) ? "申请人" : "被申请人";
        }
        return maping.get(flag);
    }

    /**
     * 开庭当事人称谓替换
     *
     * @param flag
     * @param case_no
     * @return
     */
    public static String getTitleV2(String flag, String case_no, JSONArray litigant_info) {
        String evaluate;
        if (yg_key.equals(flag)) {
            evaluate = CaseNoPartyTitleParseUtilCopy.evaluate(case_no, JSON.toJSONString(litigant_info), null);
        } else {
            evaluate = CaseNoPartyTitleParseUtilCopy.evaluate(case_no, null, JSON.toJSONString(litigant_info));
        }

        String titleOriginal = JSON.parseArray(evaluate)
                .toJavaList(JSONObject.class)
                .stream()
                .findFirst()
                .orElse(new JSONObject()).getString("title_original");
        return titleOriginal;
    }


    /**
     * 增加律师
     *
     * @param name
     * @param lawyerMapping
     * @return
     */
    public static List<JSONObject> addLawyer(List<JSONObject> name, Map<String, JSONArray> lawyerMapping) {
        if (null == name || name.isEmpty()) {
            return name;
        }

        List<JSONObject> re = name
                .stream()
                .filter(j -> j.containsKey("name"))
                .peek(j -> {
                    JSONArray l = lawyerMapping.getOrDefault(j.getString("name"), new JSONArray());
                    j.put("lawyer_info", l);
                }).collect(Collectors.toList());

        return re;

    }

    public static Map<String, JSONArray> mergeLawyers(List<BaseCaseInfo> data) {
        Map<String, JSONArray> m1 = new HashMap<>();
        if (data == null || data.isEmpty()) return m1;
        data.stream()
                .filter(f -> "0".equals(f.getFlag()))
                .forEach(x -> {
                    JSONArray plaintiff_info = x.getYg_name();
                    JSONArray defendant_info = x.getBg_name();
                    List<JSONObject> list = new ArrayList<>();
                    if (plaintiff_info != null && !plaintiff_info.isEmpty()) {
                        list.addAll(plaintiff_info.toJavaList(JSONObject.class));
                    }
                    if (defendant_info != null && !defendant_info.isEmpty()) {
                        list.addAll(defendant_info.toJavaList(JSONObject.class));
                    }
                    list.stream()
                            .filter(j -> j.containsKey("name") && j.containsKey("lawyer_info"))
                            .forEach(j -> m1.put(j.getString("name"), j.getJSONArray("lawyer_info")));

                });
        return m1;
    }

    public static String getKey(JSONObject j, String key) {
        if (j == null) return "";
        String r = j.getString(key);
        return r == null ? "" : r;
    }

    public static void main(String[] args) {
        String jsons = "[{\"flag\":\"0\",\"detail_id\":\"0020e1d517d4151f9bee1c14a9766e94\",\"yg_name\":\"[{\\\"lawyer_info\\\":[{\\\"lawfirm_id\\\":\\\"10b9dd80902c54c739381e9d9113b98e\\\",\\\"lawfirm_name\\\":\\\"江苏朗宁律师事务所\\\",\\\"lawyer_id\\\":\\\"e3d66ae10df51c347fb011ac385fe0b1\\\",\\\"lawyer_name\\\":\\\"潘云春\\\"}],\\\"litigant_id\\\":\\\"6ef6937a529779a4fa140396eea67d02\\\",\\\"name\\\":\\\"盐城仁者置业有限公司\\\",\\\"party_title\\\":\\\"申请执行人\\\"}]\",\"bg_name\":\"[{\\\"lawyer_info\\\":[{\\\"lawfirm_id\\\":\\\"93b0d9dde35e7f88e09363b5e33ea9f8\\\",\\\"lawfirm_name\\\":\\\"江苏行真律师事务所\\\",\\\"lawyer_id\\\":\\\"0cfacc2ea9b8323290fc08f89e5557d7\\\",\\\"lawyer_name\\\":\\\"陈跃祥\\\"}],\\\"litigant_id\\\":\\\"21b7eb3c3fc87a328c70bcb0ae5908f3\\\",\\\"name\\\":\\\"盐城市城市建设投资集团房地产开发有限公司\\\",\\\"party_title\\\":\\\"被执行人\\\"}]\",\"case_no\":\"(2020)苏0902执2215号\",\"case_reason\":\"商品房委托代理销售合同纠纷\",\"party_title\":\"[{\\\"judge_tendency\\\":\\\"获得支持\\\",\\\"litigant_id\\\":\\\"6ef6937a529779a4fa140396eea67d02\\\",\\\"name\\\":\\\"盐城仁者置业有限公司\\\",\\\"party_title\\\":\\\"申请执行人\\\"},{\\\"judge_tendency\\\":\\\"被查冻扣\\\",\\\"litigant_id\\\":\\\"21b7eb3c3fc87a328c70bcb0ae5908f3\\\",\\\"name\\\":\\\"盐城市城市建设投资集团房地产开发有限公司\\\",\\\"party_title\\\":\\\"被执行人\\\"}]\"},{\"flag\":\"1\",\"detail_id\":\"0713f7d00c74516a1e855aec32955975\",\"yg_name\":\"[{\\\"litigant_id\\\":\\\"6ef6937a529779a4fa140396eea67d02\\\",\\\"name\\\":\\\"盐城仁者置业有限公司\\\",\\\"party_title\\\":\\\"上诉人\\\"}]\",\"bg_name\":\"[{\\\"litigant_id\\\":\\\"21b7eb3c3fc87a328c70bcb0ae5908f3\\\",\\\"name\\\":\\\"盐城市城市建设投资集团房地产开发有限公司\\\",\\\"party_title\\\":\\\"被上诉人\\\"}]\",\"case_no\":\"(2020)苏09民终2294号\",\"case_reason\":\"商品房委托代理销售合同纠纷\",\"party_title\":null},{\"flag\":\"8\",\"detail_id\":\"c917757a78d982e9688395e084816508\",\"yg_name\":\"[{\\\"litigant_id\\\":\\\"6ef6937a529779a4fa140396eea67d02\\\",\\\"name\\\":\\\"盐城仁者置业有限公司\\\",\\\"party_title\\\":\\\"上诉人\\\"}]\",\"bg_name\":\"[{\\\"litigant_id\\\":\\\"21b7eb3c3fc87a328c70bcb0ae5908f3\\\",\\\"name\\\":\\\"盐城市城市建设投资集团房地产开发有限公司\\\",\\\"party_title\\\":\\\"被上诉人\\\"}]\",\"case_no\":\"(2020)苏09民终2294号\",\"case_reason\":\"商品房委托代理销售合同纠纷\",\"party_title\":null},{\"flag\":\"0\",\"detail_id\":\"858b1fd5b1d1b0bdd40f16ac33713c07\",\"yg_name\":\"[{\\\"litigant_id\\\":\\\"6ef6937a529779a4fa140396eea67d02\\\",\\\"name\\\":\\\"盐城仁者置业有限公司\\\",\\\"party_title\\\":\\\"上诉人（原审原告）\\\"}]\",\"bg_name\":\"[{\\\"litigant_id\\\":\\\"21b7eb3c3fc87a328c70bcb0ae5908f3\\\",\\\"name\\\":\\\"盐城市城市建设投资集团房地产开发有限公司\\\",\\\"party_title\\\":\\\"被上诉人（原审被告）\\\"}]\",\"case_no\":\"(2020)苏09民终2294号\",\"case_reason\":\"商品房委托代理销售合同纠纷\",\"party_title\":\"[{\\\"judge_tendency\\\":\\\"不被支持\\\",\\\"litigant_id\\\":\\\"6ef6937a529779a4fa140396eea67d02\\\",\\\"name\\\":\\\"盐城仁者置业有限公司\\\",\\\"party_title\\\":\\\"上诉人（原审原告）\\\"},{\\\"judge_tendency\\\":\\\"对方不被支持\\\",\\\"litigant_id\\\":\\\"21b7eb3c3fc87a328c70bcb0ae5908f3\\\",\\\"name\\\":\\\"盐城市城市建设投资集团房地产开发有限公司\\\",\\\"party_title\\\":\\\"被上诉人（原审被告）\\\"}]\"},{\"flag\":\"1\",\"detail_id\":\"eb2cb11cdd449156d035ff5480515205\",\"yg_name\":\"[{\\\"litigant_id\\\":\\\"6ef6937a529779a4fa140396eea67d02\\\",\\\"name\\\":\\\"盐城仁者置业有限公司\\\",\\\"party_title\\\":\\\"原告\\\"}]\",\"bg_name\":\"[{\\\"litigant_id\\\":\\\"21b7eb3c3fc87a328c70bcb0ae5908f3\\\",\\\"name\\\":\\\"盐城市城市建设投资集团房地产开发有限公司\\\",\\\"party_title\\\":\\\"被告\\\"}]\",\"case_no\":\"(2017)苏0902民初2126号\",\"case_reason\":null,\"party_title\":null},{\"flag\":\"0\",\"detail_id\":\"5f7c04c6e72124304460fdd146d2b310\",\"yg_name\":\"[{\\\"litigant_id\\\":\\\"6ef6937a529779a4fa140396eea67d02\\\",\\\"name\\\":\\\"盐城仁者置业有限公司\\\",\\\"party_title\\\":\\\"原告\\\"}]\",\"bg_name\":\"[{\\\"litigant_id\\\":\\\"21b7eb3c3fc87a328c70bcb0ae5908f3\\\",\\\"name\\\":\\\"盐城市城市建设投资集团房地产开发有限公司\\\",\\\"party_title\\\":\\\"被告\\\"}]\",\"case_no\":\"(2017)苏0902民初2126号\",\"case_reason\":\"商品房委托代理销售合同纠纷\",\"party_title\":\"[{\\\"judge_tendency\\\":\\\"获得部分支持\\\",\\\"litigant_id\\\":\\\"6ef6937a529779a4fa140396eea67d02\\\",\\\"name\\\":\\\"盐城仁者置业有限公司\\\",\\\"party_title\\\":\\\"原告\\\"},{\\\"judge_tendency\\\":\\\"获得部分支持\\\",\\\"litigant_id\\\":\\\"21b7eb3c3fc87a328c70bcb0ae5908f3\\\",\\\"name\\\":\\\"盐城市城市建设投资集团房地产开发有限公司\\\",\\\"party_title\\\":\\\"被告\\\"}]\"},{\"flag\":\"7\",\"detail_id\":\"596c12d11829d2936cecb292d9d1fa0b\",\"yg_name\":\"[]\",\"bg_name\":\"[{\\\"litigant_id\\\":\\\"21b7eb3c3fc87a328c70bcb0ae5908f3\\\",\\\"name\\\":\\\"盐城市城市建设投资集团房地产开发有限公司\\\",\\\"party_title\\\":\\\"被执行人\\\"}]\",\"case_no\":\"(2020)苏0902执2215号\",\"case_reason\":null,\"party_title\":null},{\"flag\":\"8\",\"detail_id\":\"c200a219cbad3c9666d45114e7ccba66\",\"yg_name\":\"[{\\\"litigant_id\\\":\\\"6ef6937a529779a4fa140396eea67d02\\\",\\\"name\\\":\\\"盐城仁者置业有限公司\\\",\\\"party_title\\\":\\\"申请执行人\\\"}]\",\"bg_name\":\"[{\\\"litigant_id\\\":\\\"21b7eb3c3fc87a328c70bcb0ae5908f3\\\",\\\"name\\\":\\\"盐城市城市建设投资集团房地产开发有限公司\\\",\\\"party_title\\\":\\\"被执行人\\\"}]\",\"case_no\":\"(2020)苏0902执2215号\",\"case_reason\":\"合同纠纷案件执行\",\"party_title\":null}]";
        String jsons1 = "[{\"flag\":\"0\",\"detail_id\":\"00f4f1e8a9ff98f2aa0e37c1198f21a7\",\"yg_name\":\"[]\",\"bg_name\":\"[{\\\"lawyer_info\\\":[],\\\"litigant_id\\\":\\\"\\\",\\\"name\\\":\\\"孙海波\\\",\\\"party_title\\\":\\\"被执行人\\\"}]\",\"case_no\":\"(2022)吉2401执3910号\",\"case_reason\":\"罚金\",\"party_title\":\"[{\\\"judge_tendency\\\":\\\"\\\",\\\"litigant_id\\\":\\\"\\\",\\\"name\\\":\\\"孙海波\\\",\\\"party_title\\\":\\\"被执行人\\\"}]\"},{\"flag\":\"8\",\"detail_id\":\"400bfa0d3b13c8d6a8b7142c37179b19\",\"yg_name\":\"[]\",\"bg_name\":\"[{\\\"litigant_id\\\":\\\"\\\",\\\"name\\\":\\\"孙海波\\\",\\\"party_title\\\":\\\"被执行人\\\"}]\",\"case_no\":\"(2022)吉2401执3910号\",\"case_reason\":\"罚金\",\"party_title\":null}]";
        get_names n = new get_names();
        System.out.println(n.evaluate(jsons));
        System.out.println(n.evaluate(jsons1));
    }

}


